# wwopenspec java demo

## demo目录结构说明

- SpecDemo: SpecSDK的demo项目，依赖fastjson和netty
- SpecSDKDemo-with-dependencies.jar: SpecSDK的demo可执行文件，构建时已附带依赖，可以独立运行
- libWeWorkSpecSDK.so: sdk需要的动态链接库，通过JNI调用
- README_DEMO.md: 本说明
- Dockerfile: 构建专区程序镜像的脚本

## 源码包结构说明

SpecDemo/src/main/java/
├── com
│   └── tencent
│       └── wework                      注意：需保持com.tencent.wework结构
│           ├── SpecCallbackSDK.java    SpecCallbackSDK接口
│           ├── SpecSDK.java            SpecSDK接口
│           └── SpecUtil.java           SDK内的通用工具
└── mytype
    └── mycom
        └── mygroup
            ├── CommonUtil.java         通用工具类
            ├── DataBaseUtils.java      存储工具类，包含简单的文件IO和一个定期清理过期文件的静态线程
            ├── NetworkService.java     网络服务类，使用Netty搭建
            ├── ResourceMonitor.java    资源监控类，定时输出JMX收集的资源使用情况
            ├── SpecDemo.java           服务启动器类，包含服务启动和配置初始化逻辑
            ├── ThreadPoolSingleton.java           自定义业务线程池单例
            ├── UserLogicHandler.java              用户业务逻辑抽象接口(1.4.0版本新增)
            ├── DemoCallProgramHandler.java        应用同步调用专区请求业务逻辑处理demo类(1.4.0版本新增)
            ├── DemoReceiveCallBackHandler.java    应用通知专区请求业务逻辑处理demo类(1.4.0版本新增)
            ├── RequestContext.java                请求上下文类(1.4.0版本新增)
            ├── RequestProcessor.java              Http请求处理类(1.4.0版本新增)
            └── SvrConfig.java                     服务相关配置类(1.4.0版本新增)



## 运行环境配置

- 使用时应将动态链接库拷贝到Java查找本地库的路径（`java.library.path`）下（如`/usr/lib`），或添加本地库查找路径
    - 本地库查找路径可在Java程序内调用`System.getProperty("java.library.path")`或命令行界面调用`java -XshowSettings:properties -version`查看
  
- sdk依赖`openssl3`，开发者请自行下载最新版。需要`libcrypto.so.3`和`libssl.so.3`，配置参考如下
    - 源码安装：进入openssl目录，构建需要的两个库：`make libcrypto.so`、`make libssl.so`。将so放入本地库加载路径，您的本地库加载路径可通过`cat /etc/ld.so.conf`查看。或者修改您的环境变量`LD_LIBRARY_PATH`添加动态链接库的查找路径
    - 包管理安装：使用您镜像的包管理下载安装x86_64的openssl3即可

## 启动命令行参数
    usage: Options（以下参数非必须，若未传则使用默认值，若需要开启调试模式，则必须传-d跟-a参数）
    -d <arg>   debugToken 调试模式时必须传该参数
    -a <arg>   accessToken 调试模式时必须传该参数
    -i <arg>   io worker thread count NioEventLoop worker线程池大小，默认为256。若processInIoThread为0，io线程建议调小
    -p <arg>   processInIoThread 业务逻辑是否在io线程执行。1:使用io线程执行业务逻辑 0: 使用自定义业务线程执行业务逻辑，默认为1
    -b <arg>   business worker thread count  自定义业务worker线程池大小，默认为256，仅当processInIoThread为1时生效
    -m <arg>   monitor interval 监控线程打日志时间间隔，默认为60s


## 其他说明

- `.so`是类Unix系统（如Linux）的动态链接库，只能在类Unix系统使用，Windows系统（`.dll`）和Mac系统（`.dylib`）的动态链接库将在后续推出，敬请期待
- **需保持包结构**，不要将sdk的源文件复制到项目包中，否则JNI的本地方法引用会失效
    - 原因：JNI注册的方法包含包结构，详见javah生成本地方法头文件