package mytype.mycom.mygroup;

import com.tencent.wework.SpecUtil;

import java.nio.charset.Charset;

/**
 * @usage 不含第三方依赖的通用工具类，提供接口如下
 *        1. 查询接口是否支持
 *        2. 生成包裹错误信息的json string
 *        3. 展示部分系统参数
 */
public class CommonUtils {
    /**
     * @usage demo内部统一错误码，用户可根据需要自定义
     */
    private static final int DEMO_INTERNAL_ERRORCODE = 710660;

    /**
     * @usage 统一错误信息格式，注意错误字段应在输出协议中注册
     */
    private static final String ERROR_RESPONSE_FORMAT = "{\"errcode\":%d,\"errmsg\":\"%s,%s,%s\"}";

    /**
     * @usage 将错误信息包装为统一形式返回，以通过平台对用户注册能力的输出协议校验
     * @param errMsg 内部错误提示信息
     * @return 包装好的错误信息，json格式，errcode为统一的demo内部错误码，errmsg为文件名、行号和错误信息
     */
    public static String getErrorResponse(String errMsg) {
        StackTraceElement element = Thread.currentThread().getStackTrace()[3];
        SpecUtil.SpecLogNative(
            'E', 
            element.getFileName(), 
            element.getLineNumber(), 
            errMsg
        );
        return String.format(
            ERROR_RESPONSE_FORMAT,
            DEMO_INTERNAL_ERRORCODE,
            element.getFileName(),
            element.getLineNumber(),
            errMsg
        );
    }

    /**
     * @usage 展示JVM关于字符编码的配置
     */
    public static void displayJvmCharsetConfig() {
        StringBuffer sb = new StringBuffer();
        sb.append("Default Charset: " + Charset.defaultCharset())
            .append(", System Charset: " + System.getProperty("file.encoding"))
            .append(", Default Charset in Use: " + new java.io.OutputStreamWriter(new java.io.ByteArrayOutputStream()).getEncoding())
            .append(", Default OS Charset: " + System.getProperty("sun.jnu.encoding"))
            .append(", Default Locale Charset: " + java.nio.charset.Charset.defaultCharset());
        SpecUtil.WWSpecLogInfo(sb.toString());
    }

    /**
     * @usage 私有构造函数，防止被实例化
     */
    private CommonUtils() {};
}
