package mytype.mycom.mygroup;

import com.tencent.wework.SpecCallbackSDK;
import com.tencent.wework.SpecSDK;

/*
 * @usage 应用调用专区接口的handler示例Demo类
 */
public class DemoCallProgramHandler implements  UserLogicHandler{


    /*
     * @usage 透传请求的能力ID标识
     */
    private static final String TRANSMIT_SDK_PREFIX = "invoke_";

    /*
     * @usage 企业应用来获取回调事件数据的请求的能力ID
     */
    private static final String ABILITY_GET_CALLBACK_DATA = "get_callback_data";

    @Override
    public boolean isValidAblility(String abilityId){
        return abilityId.startsWith(TRANSMIT_SDK_PREFIX) || ABILITY_GET_CALLBACK_DATA.equals(abilityId);
    }

    @Override
    public String process(SpecCallbackSDK callback){
        String abilityId = callback.GetAbilityId();
        if(!isValidAblility(abilityId)) {
            return CommonUtils.getErrorResponse("unknown abilityId: " + abilityId);
        }

        if (abilityId.startsWith(TRANSMIT_SDK_PREFIX)) {
            return transmitRequest(callback, abilityId.substring(7));
        } else if (ABILITY_GET_CALLBACK_DATA.equals(abilityId)) {
            return getCallbackData(callback);
        } else {
            return CommonUtils.getErrorResponse("unknown abilityId: " + abilityId);
        }
    }

    /*
     * @usage 透传企业应用的请求，转发到企微专区后台，相当于企业应用的代理
     * @param callback 企业应用的请求
     * @return sdk的回包
     */
    private static String transmitRequest(SpecCallbackSDK callback, String apiName) {
        SpecSDK sdk = new SpecSDK(callback);
        sdk.SetRequest(callback.GetData());

        int ret = sdk.Invoke(apiName);
        if (ret != 0) {
            return CommonUtils.getErrorResponse("invoke failed, ret = " + ret + ", api name = " + apiName);
        }

        return sdk.GetResponse();
    }

    /*
     * @usage 从专区程序本地读取回调事件的数据
     * @param callback 企业应用的请求
     * @return 回调事件的数据
     */
    private static String getCallbackData(SpecCallbackSDK callback) {
        String notifyId = callback.GetNotifyId();
        if (notifyId.isEmpty()) {
            return CommonUtils.getErrorResponse("notify_id is empty");
        }

        StringBuffer dataBuffer = new StringBuffer();
        DataBaseUtils.ErrorCode errorCode = DataBaseUtils.getByNotifyId(notifyId, dataBuffer);
        if (errorCode != DataBaseUtils.ErrorCode.SUCCESS) {
            return CommonUtils.getErrorResponse("get data by notify_id failed, errorCode = " + errorCode);
        }

        return dataBuffer.toString();
    }
}