package mytype.mycom.mygroup;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONValidator;
import com.tencent.wework.SpecCallbackSDK;
import com.tencent.wework.SpecSDK;
import com.tencent.wework.SpecUtil;

public class DemoReceiveCallBackHandler implements UserLogicHandler{
    /*
     * @usage 企微专区后台回包的错误码的key
     */
    private static final String QW_BACKEND_ERRCODE_KEY = "errcode";

    @Override
    public boolean isValidAblility(String abilityid){
        return true;
    }

    @Override
    public String process(SpecCallbackSDK callback){
        return backendCallbackEvent(callback);
    }

    /*
     * @param callback 企微专区后台的回调事件
     * @return 响应消息体，形式为json string
     * @description 企业微信回调事件, 暂存回调数据，产生"program_notify"事件通知应用主动来获取，详见：
     *              https://developer.work.weixin.qq.com/document/path/99843
     *              https://developer.work.weixin.qq.com/document/path/99992
     * @note 1. sdk version 1.0.6新增特性：spec notify app支持自定义notify id，
     *          用户可使用SpecUtil.GenerateNotifyId生成notify idh或自定义notify id
     *          先将其与回调数据作关联存储，再调用spec notify app将该notify id递送到企业应用
     *       2. 用于仍可以按照原版本：先通知企业应用，再将回包的notify id与回调事件数据作关联存储
     *          专区环境能保证专区程序比企业应用早收到回包，调试模式下可能出现数据还未写入完毕就被请求的情况
     */
    private static String backendCallbackEvent(SpecCallbackSDK callback) {
        // 生成notify id，将其和回调事件的数据作关联存储
        String notifyId = SpecUtil.GenerateNotifyId();
        DataBaseUtils.ErrorCode errorCode = DataBaseUtils.setNotifyData(notifyId, callback.GetData());
        if (errorCode != DataBaseUtils.ErrorCode.SUCCESS) {
            return CommonUtils.getErrorResponse("notify data store failed, errorCode = " + errorCode);
        }

        // 存储完毕后，调用sdk通知企业应用来获取回调事件的数据
        SpecSDK sdk = new SpecSDK(callback);
        sdk.SetRequest(String.format("{\"notify_id\": \"%s\"}", notifyId));
        int ret = sdk.Invoke("spec_notify_app");
        if (ret != 0) {
            return CommonUtils.getErrorResponse("invoke failed, ret = " + ret);
        }
        SpecUtil.WWSpecLogInfo("invoke spec notify app done", "notify id = " + notifyId);

        // 检验回包参数
        String responseStr = sdk.GetResponse();
        if (!JSONValidator.from(responseStr).validate() || responseStr.isEmpty()) {
            return CommonUtils.getErrorResponse("notifyRsp is invalid, notifyRsp = " + responseStr);
        }
        JSONObject responseJson = JSON.parseObject(responseStr);
        if (!responseJson.containsKey(QW_BACKEND_ERRCODE_KEY)) {
            return CommonUtils.getErrorResponse("missing errcode in notifyRsp");
        }
        Integer errcode = responseJson.getInteger("errcode");
        if (errcode != 0) {
            return CommonUtils.getErrorResponse("invoke spec notify app failed, errcode = " + errcode + ", errmsg = " + responseJson.getString("errmsg"));
        }

        // 这里不需要包装，回调事件无需发送回包
        return "notify and store success";
    }
}