package mytype.mycom.mygroup;

import com.tencent.wework.SpecUtil;
import org.apache.commons.cli.*;

import java.io.PrintStream;

/**
 * @note 开发调试前必读：
 * 1. 如需用于生产环境，建议完善程序的可靠性
 * 2. 启动app需指定字符编码为UTF-8：java -Dfile.encoding=UTF-8 -jar myapp.jar
 * 3. 调试前请先阅读各public类的注释，了解demo server和SDK的基本框架和逻辑功能
 * 4. 如需打日志，必须封装SpecUtil.SpecLog或SpecUtil.SpecLogNative方法
 * 5. 所有demo源代码和SDK包含大量中文注释，源文件的字符编码均为UTF-8
 * @expamle 调试前请先查看下列示例：
 * 1. 应用使用token获取会话记录全流程：https://developer.work.weixin.qq.com/document/path/100052#应用获取会话记录的流程示例
 * 2. 专区程序示例镜像配置说明：https://developer.work.weixin.qq.com/document/54166
 */
public class SpecDemo {

    /**
     * @usage 1. 设置全局字符编码为UTF-8（企微专区后台为UTF-8）
     *        2. 加载DatabaseUtil，启动过期文件清除线程
     *        3. 检查部分关键配置
     */
    static {
        try {
            System.setOut(new PrintStream(System.out, true, "UTF-8"));
            System.setErr(new PrintStream(System.err, true, "UTF-8"));
            System.setProperty("file.encoding", "UTF-8");
            // Java Native Interface (JNI) 使用的编码方式
            System.setProperty("sun.jnu.encoding", "UTF-8");
        } catch (Exception e) {
            SpecUtil.WWSpecLogError("charset init error");
            e.printStackTrace();
            System.exit(1);
        }

        try {
            Class.forName("mytype.mycom.mygroup.DataBaseUtils");
        } catch (ClassNotFoundException e) {
            SpecUtil.WWSpecLogError("load DataBaseUtils failed");
            e.printStackTrace();
            System.exit(1);
        }

        CommonUtils.displayJvmCharsetConfig();
    }

    /**
     * @usage 解析入参，处理进程级调用，启动服务器，启动调试模式的参数如下：
     */
    public static void main(String[] args) {
        // CommandLineParser parser = new DefaultParser(args);
        CommandLineParser parser = new DefaultParser();
        HelpFormatter formatter = new HelpFormatter();
        Options options = new Options();
        options.addOption("d", true, "debugToken 调试模式时必须传该参数");
        options.addOption("a", true, "accessToken 调试模式时必须传该参数");
        options.addOption("i", true, "io worker thread count NioEventLoop worker线程池大小，默认为256。若开启业务线程执行，io线程建议调小");
        options.addOption("b", true, "business worker thread count  自定义业务worker线程池大小，默认为256，仅当processInIoThread为1时生效");
        options.addOption("p", true, "processInIoThread 业务逻辑是否在io线程执行。1:使用io线程执行业务逻辑 0: 使用自定义业务线程执行业务逻辑，默认为1");
        options.addOption("m", true, "monitor interval 监控线程打日志时间间隔，默认为60s");

        try {
            CommandLine cmd = parser.parse(options, args);
            if (cmd.hasOption("i")) {
                SvrConfig.io_gorup_size = Integer.parseInt(cmd.getOptionValue("i"));
            }

            if (cmd.hasOption("b")) {
                SvrConfig.business_gorup_size = Integer.parseInt(cmd.getOptionValue("b"));
            }

            if (cmd.hasOption("p")) {
                SvrConfig.process_in_iothread = Integer.parseInt(cmd.getOptionValue("p")) != 0;
                if(!SvrConfig.process_in_iothread)
                {
                    ThreadPoolSingleton.initialize(SvrConfig.business_gorup_size);
                }
            }

            if (cmd.hasOption("m")) {
                SvrConfig.monitor_interval = Integer.parseInt(cmd.getOptionValue("m"));
            }

            if (cmd.hasOption("d")) {
                if (!cmd.hasOption("a")) {
                    SpecUtil.WWSpecLogError("cmdline miss -a access_token");
                    System.exit(1);
                }
                if (!SpecUtil.SpecOpenDebugMode(cmd.getOptionValue("d"), cmd.getOptionValue("a"))) {
                    SpecUtil.WWSpecLogError("open debug mode failed");
                    System.exit(1);
                }
            }

            try {
                NetworkService.startServer();
            } catch (Exception e) {
                SpecUtil.WWSpecLogError("start server failed" + e.getStackTrace());
                formatter.printHelp("Options", options);
                e.printStackTrace();
            }



        } catch (ParseException e) {
            SpecUtil.WWSpecLogError("cmd options err");
            formatter.printHelp("Options", options);
            System.exit(1);
        }

    }
}
