package mytype.mycom.mygroup;

import java.util.concurrent.*;

/*
 * @usage 自定义业务线程池单例
 * */
public class ThreadPoolSingleton {

    private ThreadPoolSingleton() {}

    // 静态内部类
    private static class Holder {
        private static ThreadPoolExecutor instance_;

        private static void initialize(int poolSize) {
            if (instance_ == null) {
                instance_ = new ThreadPoolExecutor(32, poolSize, 1000, TimeUnit.SECONDS, new ArrayBlockingQueue<Runnable>(16));
            }
        }
    }

    public static void initialize(int poolSize) {
        Holder.initialize(poolSize);
    }

    public static ThreadPoolExecutor getInstance() {
        if (Holder.instance_ == null) {
            throw new IllegalStateException("ThreadPoolExecutor is not initialized. Call initialize() first.");
        }
        return Holder.instance_;
    }
}
